-module(index).

-include_lib("kvs/include/entry.hrl").
-include_lib("nitro/include/nitro.hrl").
-include_lib("n4u/include/wf.hrl").


-export([
  main/0,
  event/1,
  loop/1
]).


main() ->
  case wf:user() of
    undefined ->
      wf:redirect({http, "login.html"}),
      [];
    _ ->
      #dtl{file = "index", app = n2o_sample, bindings = [
        {body, body()},
        {list, list()}
      ]}
  end.

list() -> "".
code() ->
  case wf:q(<<"code">>) of
    undefined -> "../privacy.html";
    Code -> wf:to_list(wf:depickle(Code))
  end.


body() ->
  wf:update(heading,  #b{id=heading, body="Review: " ++ code()}),
  wf:update(logout,   #button{id=logout, body="Logout " ++ wf:user(), postback=logout}),
  [ #span{id=upload}, #button{id=send, body= <<"Chat">>, postback=chat, source=[message] } ].

event(init) ->
  %%io:format("~p~n", [wf:config(n4u, appurl, undefined)]),
  
  Room = code(),
  wf:update(upload, #upload{id=upload}),
  wf:reg(n2o_session:session_id()),
  wf:reg({topic, Room}),
  Res = wf:async("looper", fun index:loop/1),
  n2o_async:send("looper", "waterline"),
  wf:info(?MODULE, "Async Process Created: ~p at Page Pid ~p~n", [Res, self()]),
  [ event({client, {E#entry.from, E#entry.media}}) || E <- kvs:entries(kvs:get(feed, {room, Room}), entry, 10) ];

event(logout) ->
  wf:logout(),
  wf:redirect("login.htm");

event(chat) ->
  User = wf:user(),
  Message = wf:q(message),
  wf:info(?MODULE, "Chat pressed: ~p~n", [Message]),
  Room = code(),
  kvs:add(#entry{id=kvs:next_id("entry", 1), from=wf:user(), feed_id={room, Room}, media=Message}),
  wf:send({topic, Room}, #client{data={User, Message}});

event(#client{data={User, Message}}) ->
  wf:wire(#jq{target=message, method=[focus, select]}),
  HTML = wf:to_list(Message),
  wf:info(?MODULE, "HTML: ~tp~n", [HTML]),
  DTL = #dtl{file="message", app=n2o_sample, bindings=[{user, User}, {color, "gray"}, {message, HTML}]},
  wf:insert_top(history, wf:jse(wf:render(DTL)));

event(#bin{data=Data}) ->
  wf:info(?MODULE, "Binary Delivered ~p~n", [Data]),
  #bin{data = "SERVER"};

event(#ftp{sid=Sid, filename=Filename, status={event, stop}}=Data) ->
  wf:info(?MODULE, "FTP Delivered ~p~n", [Data]),
  Name = hd(lists:reverse(string:tokens(wf:to_list(Filename), "/"))),
  erlang:put(message, wf:render(#link{href=iolist_to_binary(["/static/", Sid, "/", wf:url_encode(Name)]), body=Name})),
  wf:info(?MODULE, "Message ~p~n", [wf:q(message)]),
  event(chat);

event(Event) ->
  wf:info(?MODULE,"Event: ~p", [Event]),
  ok.

loop(M) ->
  DTL = #dtl{file="message", app=n2o_sample, bindings=[
    {user, "system"},
    {message, M},
    {color, "silver"}
  ]},
  wf:insert_top(history, wf:jse(wf:render(DTL))),
  wf:flush().

