-module(pgm).

%% postgresql conn and queries wrappers module


-export([
  mypg/0,        %% without pool
  transaction/2, %% without pool
  select/3,      %% without pool
  in_up_del/3,   %% without pool
  returning/3,   %% without pool
  
  transaction/1,   %% with pool
  transaction_q/3, %% with pool
  select/2,        %% with pool
  in_up_del/2,     %% with pool
  returning/2      %% with pool
]).


mypg() ->
  %{ok, C} = epgsql:connect("localhost", "user", "pass", [{database, "test"}, {port, 5432}, {timeout, 2000}]), %% can be used without connection pool
  Conn = epgsql:connect(
    application:get_env(n4u, pgs_host, "localhost"), %% can be used without connection pool
    application:get_env(n4u, pgs_user, "user"),      %% gets settings from sys.config
    application:get_env(n4u, pgs_pass, "pass"),
    [{database, application:get_env(n4u, pgs_db, "test")},
     {port, application:get_env(n4u, pgs_port, 6432)},
     {timeout, 3000}]),
  case Conn of
    {ok, C} ->
      C;
    {error, M} ->
      io:format("Database error: ~p~n", [epgsql_errcodes:to_name(M)]),
      wf:wire(["alert('db conn error!');"]),
      %{error, M}
      error
  end.


transaction(Ppid, Fun) ->
  case epgsql:squery(Ppid, "BEGIN") of
    {ok, _, _} ->
      try
        Result = Fun(),
        epgsql:squery(Ppid, "COMMIT"),
        Result
      catch
        Err1:Err2 ->
          epgsql:squery(Ppid, "ROLLBACK"),
          {Err1, Err2}
      end;
    Error -> Error
  end.


select(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, _, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.


in_up_del(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.


returning(Ppid, Q, A) ->
  case epgsql:equery(Ppid, Q, A) of
    {ok, 1, _, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.



transaction(Fun) ->
  case epgsql_pool:transaction(my_main_pool, Fun) of
    ok ->
      ok;
    Error ->
      io:format("transaction error: ~p~n in tr fun: ~p~n", [Error, Fun]),
      Error
  end.


transaction_q(Worker, Q, A) ->
  epgsql_pool:query(Worker, Q, A).


select(Q, A) ->
  case epgsql_pool:query(my_main_pool, Q, A) of
    {ok, _, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.


in_up_del(Q, A) ->
  case epgsql_pool:query(my_main_pool, Q, A) of
    {ok, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.


returning(Q, A) ->
  case epgsql_pool:query(my_main_pool, Q, A) of
    {ok, 1, _, R} ->
      R;
    {error, E} = Err ->
      io:format("~p~n", [E]),
      Err
  end.

